Package: singleRcapture
Type: Package
Title: Single-Source Capture-Recapture Models
Version: 1.1.0
Authors@R: 
    c(person(given = "Piotr",
             family = "Chlebicki",
             role = c("aut", "ctb"),
             email = "piotrek.pecet@gmail.com",
             comment = c(ORCID = "0009-0006-4867-7434")),
      person(given = "Maciej",
             family = "Beręsewicz",
             role = c("aut", "cre"),
             email = "maciej.beresewicz@ue.poznan.pl",
             comment = c(ORCID = "0000-0002-8281-4301")),
      person(given = "Cyprian",
             family = "Jurkowski",
             role = "ctb"))
Maintainer: Maciej Beręsewicz <maciej.beresewicz@ue.poznan.pl>
Description: Implementation of single-source capture-recapture methods for population size estimation using zero-truncated, zero-one truncated and zero-truncated one-inflated Poisson, Geometric and Negative Binomial regression as well as Zelterman's, Chao's and ratio-regression estimators. Package includes point and interval estimators for the population size with variances estimated using analytical or bootstrap method. Details can be found in: van der Heijden et all. (2003) <doi:10.1191/1471082X03st057oa>, Böhning and van der Heijden (2019) <doi:10.1214/18-AOAS1232>, Böhning et al. (2020) Capture-Recapture Methods for the Social and Medical Sciences or Böhning and Friedl (2021) <doi:10.1007/s10260-021-00556-8>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: yes
RdMacros: mathjaxr
BuildManual: TRUE
VignetteBuilder: knitr
Depends: R (>= 3.5.0)
RoxygenNote: 7.3.3
URL: https://github.com/ncn-foreigners/singleRcapture
BugReports: https://github.com/ncn-foreigners/singleRcapture/issues
Imports: stats, lamW, mathjaxr, sandwich, doParallel, foreach, parallel
Suggests: rmarkdown, knitr, tinytest, covr, VGAM, lmtest
NeedsCompilation: no
Packaged: 2026-05-13 07:34:02 UTC; berenz
Author: Piotr Chlebicki [aut, ctb] (ORCID:
    <https://orcid.org/0009-0006-4867-7434>),
  Maciej Beręsewicz [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8281-4301>),
  Cyprian Jurkowski [ctb]
Repository: CRAN
Date/Publication: 2026-05-13 08:30:02 UTC
Built: R 4.5.2; ; 2026-05-13 11:18:14 UTC; unix
