# Generated by roxygen2: do not edit by hand

S3method(add1,singleRStaticCountData)
S3method(anova,singleRStaticCountData)
S3method(bread,singleRStaticCountData)
S3method(coef,singleRRatioReg)
S3method(confint,singleRRatioReg)
S3method(confint,singleRStaticCountData)
S3method(cooks.distance,singleRStaticCountData)
S3method(df.residual,singleRStaticCountData)
S3method(dfbeta,singleRStaticCountData)
S3method(dfpopsize,singleRStaticCountData)
S3method(drop1,singleRStaticCountData)
S3method(estfun,singleRStaticCountData)
S3method(estimatePopsize,default)
S3method(extractAIC,singleRRatioReg)
S3method(extractAIC,singleRStaticCountData)
S3method(family,singleRStaticCountData)
S3method(fitted,singleRStaticCountData)
S3method(hatvalues,singleRStaticCountData)
S3method(influence,singleRStaticCountData)
S3method(logLik,singleRRatioReg)
S3method(logLik,singleRStaticCountData)
S3method(model.frame,singleRStaticCountData)
S3method(model.matrix,singleRStaticCountData)
S3method(nobs,singleRRatioReg)
S3method(nobs,singleRStaticCountData)
S3method(plot,singleRRatioReg)
S3method(plot,singleRStaticCountData)
S3method(popSizeEst,singleRRatioReg)
S3method(popSizeEst,singleRStaticCountData)
S3method(predict,singleRStaticCountData)
S3method(print,popSizeEstResults)
S3method(print,singleRRatioReg)
S3method(print,singleRStaticCountData)
S3method(print,singleRfamily)
S3method(print,summarysingleRRatioReg)
S3method(print,summarysingleRStaticCountData)
S3method(print,summarysingleRmargin)
S3method(profile,singleRStaticCountData)
S3method(redoPopEstimation,singleRStaticCountData)
S3method(residuals,singleRStaticCountData)
S3method(rstandard,singleRStaticCountData)
S3method(rstudent,singleRStaticCountData)
S3method(sigma,singleRStaticCountData)
S3method(simulate,singleRStaticCountData)
S3method(simulate,singleRfamily)
S3method(stratifyPopsize,singleRStaticCountData)
S3method(summary,singleRRatioReg)
S3method(summary,singleRStaticCountData)
S3method(summary,singleRmargin)
S3method(vcov,singleRStaticCountData)
S3method(vcovHC,singleRStaticCountData)
export(Hurdleztgeom)
export(Hurdleztnegbin)
export(Hurdleztpoisson)
export(chao)
export(controlMethod)
export(controlModel)
export(controlPopVar)
export(dfpopsize)
export(estimatePopsize)
export(estimatePopsizeFit)
export(marginalFreq)
export(oichao)
export(oiztgeom)
export(oiztnegbin)
export(oiztpoisson)
export(popSizeEst)
export(ratioReg)
export(redoPopEstimation)
export(stratifyPopsize)
export(zelterman)
export(zotgeom)
export(zotnegbin)
export(zotpoisson)
export(ztHurdlegeom)
export(ztHurdlenegbin)
export(ztHurdlepoisson)
export(ztgeom)
export(ztnegbin)
export(ztoigeom)
export(ztoinegbin)
export(ztoipoisson)
export(ztpoisson)
import(mathjaxr)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,panel.smooth)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(lamW,lambertW0)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(sandwich,bread)
importFrom(sandwich,estfun)
importFrom(sandwich,vcovHC)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,add1)
importFrom(stats,anova)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,contrasts)
importFrom(stats,cooks.distance)
importFrom(stats,dbinom)
importFrom(stats,density)
importFrom(stats,df.residual)
importFrom(stats,dfbeta)
importFrom(stats,dlnorm)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,drop1)
importFrom(stats,extractAIC)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,hatvalues)
importFrom(stats,influence)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,profile)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,reformulate)
importFrom(stats,residuals)
importFrom(stats,rstandard)
importFrom(stats,rstudent)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,sigma)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,uniroot)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
