% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscDiagnostics.R
\name{confint.singleRStaticCountData}
\alias{confint.singleRStaticCountData}
\title{Confidence intervals for model parameters}
\usage{
\method{confint}{singleRStaticCountData}(object, parm, level = 0.95, ...)
}
\arguments{
\item{object}{object of singleRStaticCountData class.}

\item{parm}{names of parameters for which confidence intervals are to be
computed, if missing all parameters will be considered.}

\item{level}{confidence level for intervals.}

\item{...}{currently does nothing.}
}
\value{
An object with named columns that include upper and
lower limit of confidence intervals.
}
\description{
A function that computes studentized confidence intervals
for model coefficients.
}
